--[[
##Version: 1.1.0
##Author: kevink98
##Date: 29.11.2016
##ModName: FuelTrigger
##Desc: Milchverkauf

##Changelog: 	V 1.0.0
					- Release
				V 1.1.0
					- Fix zum Einbau mit dem GiantsEditor
					- Fix Verkaufspreis (Preis wird jetzt vom Spiel bestimmt)

]]--

 
FuelSellTrigger = {};
FuelSellTrigger.ModDir = g_currentModDirectory
FuelSellTrigger_mt = nil


local nBeginn, nEnde = string.find(FuelSellTrigger.ModDir,"placeable");
if nBeginn then
	FuelSellTrigger_mt = Class(FuelSellTrigger, Placeable);
else
	FuelSellTrigger.RunAsGE = true;
	FuelSellTrigger_mt = Class(FuelSellTrigger, Object);
end

InitObjectClass(FuelSellTrigger, "FuelSellTrigger");

function FuelSellTrigger.onCreate(id)
    local trigger = FuelSellTrigger:new(g_server ~= nil, g_client ~= nil);
	g_currentMission:addOnCreateLoadedObject(trigger);
    if trigger:load(id) then
		g_currentMission:addOnCreateLoadedObjectToSave(trigger);
        trigger:register(true);
    else
        trigger:delete();
    end
end;

function FuelSellTrigger:new(isServer, isClient, customMt)
 local mt = customMt;
    if mt == nil then
        mt = FuelSellTrigger_mt;
    end;
	local self = {};
	if FuelSellTrigger.RunAsGE then
		self = Object:new(isServer, isClient, mt)
	else
		self = Placeable:new(isServer, isClient, mt);
		registerObjectClassName(self, "FuelSellTrigger");
	end;
	self.FuelTrailers = {};
	self.FuelActivable = FuelActivable:new(self)
	self.isFilling = false;
	self.nodeId = 0;
    return self;
end;

function FuelSellTrigger:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
	self.trailerTipTrigger = {}
	
	if FuelSellTrigger.RunAsGE then
		self.saveId = getUserAttribute(xmlFilename,"saveId");
		if self.saveId == nil then
			self.saveId = "FuelSellTrigger_"..getName(xmlFilename)
		end
		self.triggerId = xmlFilename;
	end;
	
	if not self.RunAsGE then
		if not FuelSellTrigger:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
			return false;
		end;
		return true;
	else
		self.nodeId = xmlFilename;
		if not self:finalizePlacement() then return false; end;
	end;

	return true;
end;

function FuelSellTrigger:finalizePlacement(x, y, z, rx, ry, rz, initRandom)
	if not self.RunAsGE then
		FuelSellTrigger:superClass().finalizePlacement(self)
	end
	
	local fullViewName = g_i18n:getText("FuelSellTrigger_pda");
	local imageUVs = getNormalizedUVs({520, 520, 240, 240})
	local x, _, z = getWorldTranslation(self.nodeId);
	local filenames = {"dataS2/menu/hud/mapHotspots_1024.png", "dataS2/menu/hud/mapHotspots_512.png", "dataS2/menu/hud/mapHotspots_256.png", "dataS2/menu/hud/mapHotspots_128.png",  "dataS2/menu/hud/mapHotspots_64.png"}
	self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("tipTrigger", fullViewName, filenames, imageUVs, {0.9559, 0.5647, 0.0423, 1}, x, z, nil, nil, false, false, true, 0, true, MapHotspot.CATEGORY_TRIGGER)
    
	if self.isClient then
		local SoundFile  = Utils.getFilename("$data/maps/sounds/refuel.wav",  ModDir);	
		self.SourceRefuel = createAudioSource("RefuelSound", SoundFile, 30, 15, 0, 1);
		self.sampleRefuel = getAudioSourceSample(self.SourceRefuel);
		link(self.nodeId, self.SourceRefuel);
	end
	
	self.trigger = getChild(self.nodeId,"FuelTrigger")
	if self.trigger and self.trigger ~= 0 then
		self.fillType = FillUtil.fillTypeNameToInt[getUserAttribute(self.nodeId,"fillType")];
		if self.fillType ~= nil then
			addTrigger(self.trigger,"triggerCallback",self);
			self.priceMult = Utils.getNoNil(getUserAttribute(self.nodeId,"priceMult"),1.2)
			self.moneyChangeId = getMoneyTypeId()
			self.lastMoneyChange = 0;
			self.FuelActivable.textStart = Utils.getNoNil(g_i18n:getText("FuelSellTrigger_startFill"),"Verkaufen starte")
			self.FuelActivable.textEnd = Utils.getNoNil(g_i18n:getText("FuelSellTrigger_stopFill"),"Verkaufen stoppen")
		else
			return false;
		end;
	else
		return false;
	end;
	
	if not self.RunAsGE then
		
	else
		g_currentMission:addNodeObject(self.nodeId, self)
	end;
	self.FuelSellTriggerDirtyFlag = self:getNextDirtyFlag();
	return true;
end;

function FuelSellTrigger:getSaveAttributesAndNodes(nodeIdent) 
	local attributes, nodes = "","";
	if not self.RunAsGE then
		attributes, nodes = FuelSellTrigger:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	end;
	return attributes,nodes;
end

function FuelSellTrigger:loadFromAttributesAndNodes(xmlFile, key, resetVehicles) 
	if not self.RunAsGE and not FuelSellTrigger:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false
	end
	return true
end

function FuelSellTrigger:writeAllStream(streamId, connection)

end;
function FuelSellTrigger:readAllStream(streamId, connection) 

end; 
function FuelSellTrigger:writeStream(streamId, connection)
	FuelSellTrigger:superClass().writeStream(self, streamId, connection)
	self:writeAllStream(streamId, connection)
end
function FuelSellTrigger:readStream(streamId, connection)
	FuelSellTrigger:superClass().readStream(self, streamId, connection)
	self:readAllStream(streamId, connection)
end;
function FuelSellTrigger:writeUpdateStream(streamId, connection, dirtyMask)
	FuelSellTrigger:superClass().writeUpdateStream(self, streamId, connection, dirtyMask);
	self:writeAllStream(streamId, connection)
end;
function FuelSellTrigger:readUpdateStream(streamId, timestamp, connection)
	FuelSellTrigger:superClass().readUpdateStream(self, streamId, timestamp, connection);
	self:readAllStream(streamId, connection)
end;

function FuelSellTrigger:deleteMap()
	self:delete();
end;

function FuelSellTrigger:delete()
	unregisterObjectClassName(self);
	g_currentMission:removeOnCreateLoadedObjectToSave(self)
	if self.mapHotspot then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot)
	end;
	if self.trigger ~= nil then
		removeTrigger(self.trigger)
	end;
	if not self.RunAsGE then FuelSellTrigger:superClass().delete(self) end;
end;

function FuelSellTrigger:update(dt) 
	if self.isFilling then
		local disableFilling = true;
		if self.FuelFillTrailer then
			if self.isClient and self.FuelFillTrailerForPipe.setOverloadPipe ~= nil and self.FuelFillTrailerForPipe.setTrigger ~= nil then
				self.FuelFillTrailerForPipe:setTrigger(self.trigger);
				self.FuelFillTrailerForPipe:setOverloadPipe(self.trigger);
			end
			if self.isServer then
				local fillLvl = self.FuelFillTrailer:getFillLevel(self.fillType)
				local fillLitersPerSecond = self.FuelFillTrailer.fillLitersPerSecond or self.FuelFillTrailer.fuelFillLitersPerSecond or self.fillLitersPerSecond;
				local delta = math.min(fillLitersPerSecond*dt*0.001,fillLvl);
				
				if delta > 0 then
					disableFilling = false;
					self.FuelFillTrailer:setFillLevel(fillLvl-delta, self.fillType, true)
					
					local desc = FillUtil.fillTypeIndexToDesc[self.fillType]
					desc.totalAmount = desc.totalAmount + delta
					local price = delta * FillUtil.fillTypeIndexToDesc[self.fillType].pricePerLiter * EconomyManager.getPriceMultiplier()
					g_currentMission:addSharedMoney(price, "other")
					g_currentMission:addMoneyChange(price, self.moneyChangeId)
					self.lastMoneyChange = 30;
				end;
			end;
		end;
		if self.isServer and disableFilling then
			self:setIsFuelTankFilling(false)
		end;
	end;
	
	if self.lastMoneyChange > 0 then
		self.lastMoneyChange = self.lastMoneyChange - 1
        if self.lastMoneyChange == 0 then
            g_currentMission:showMoneyChange(self.moneyChangeId, g_i18n:getText("finance_soldFuel"))
        end
	end
end;

function FuelSellTrigger:setIsFuelTankFilling(isFilling, trailer,noEventSend)
	FuelSellTriggerEvent.sendEvent(self, self.triggerId, isFilling, trailer, noEventSend)
	if self.isFilling ~= isFilling then
		self.isFilling = isFilling
		self.FuelFillTrailer = trailer;
	end;
	if self.isClient then
		if self.sampleRefuel ~= nil then
			if isFilling then
				playSample(self.sampleRefuel,0,1,0)
			else
				stopSample(self.sampleRefuel)
			end
		end
		if self.FuelFillTrailerForPipe ~= nil and self.FuelFillTrailerForPipe.UpdateOverloadPipe then
			self.FuelFillTrailerForPipe.isFillingOverloadPipe = isFilling;
		end;
	end;
end;

function FuelSellTrigger:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	local trailer = Utils.getNoNil(g_currentMission.objectToTrailer[otherShapeId], g_currentMission.objectToTrailer[otherActorId]);
	if trailer ~= nil then
		if onEnter then
			if table.getn(self.FuelTrailers) == 0 then
				g_currentMission:addActivatableObject(self.FuelActivable)
			end
			table.insert(self.FuelTrailers, trailer)
		else
			for i = 1, table.getn(self.FuelTrailers), 1 do
				if self.FuelTrailers[i] == trailer then
					table.remove(self.FuelTrailers, i)
					break
				end
			end

			if table.getn(self.FuelTrailers) == 0 then
				g_currentMission:removeActivatableObject(self.FuelActivable)
			end
			
			if self.isServer and self.FuelFillTrailer == trailer then
				self:setIsFuelTankFilling(false)
			end;
		end
	end
end;


FuelActivable = {};
local FuelActivable_mt = Class(FuelActivable);
function FuelActivable:new(trigger)
	local self = {};
	
	setmetatable(self, FuelActivable_mt)

	self.FuelTrigger = trigger
	self.activateText = "unknown"
	self.currentTrailer = nil

	return self
end;

function FuelActivable:getIsActivatable()
	self.currentTrailer = nil
	
	for _, trailer in pairs(self.FuelTrigger.FuelTrailers) do
		--if trailer.getIsActiveForInput(trailer) then
			if trailer:allowFillType(self.FuelTrigger.fillType, false) then
				self.currentTrailer = trailer;
				self.FuelTrigger.FuelFillTrailerForPipe = trailer;
				self.updateActivateText(self)	
				return true
			end
		--end
	end
	return false
end
function FuelActivable:onActivateObject()
	self.FuelTrigger:setIsFuelTankFilling(not self.FuelTrigger.isFilling, self.currentTrailer)
	self.updateActivateText(self)
	g_currentMission:addActivatableObject(self)
end
function FuelActivable:drawActivate()
end
function FuelActivable:updateActivateText()
	if self.FuelTrigger.isFilling then
		self.activateText = self.textEnd;
	else
		self.activateText = self.textStart;
	end
end

if FuelSellTrigger.RunAsGE then
	g_onCreateUtil.addOnCreateFunction("FuelSellTrigger", FuelSellTrigger.onCreate);
else
	registerPlaceableType("FuelSellTrigger", FuelSellTrigger);
end;

FuelSellTriggerEvent = {}
FuelSellTriggerEvent_mt = Class(FuelSellTriggerEvent, Event)
InitEventClass(FuelSellTriggerEvent, "FuelSellTriggerEvent")
function FuelSellTriggerEvent:emptyNew()
	local self = Event:new(FuelSellTriggerEvent_mt)
	return self
end
function FuelSellTriggerEvent:new(object, triggerId, isFilling, trailer)
	local self = FuelSellTriggerEvent:emptyNew()
	self.object = object
	self.triggerId = triggerId;
	self.isFilling = isFilling
	self.trailer = trailer
	return self
end
function FuelSellTriggerEvent:readStream(streamId, connection)
	self.object = readNetworkNodeObject(streamId)
	self.triggerId = streamReadInt32(streamId);
	self.isFilling = streamReadBool(streamId)
	if self.isFilling then
		self.trailer = readNetworkNodeObject(streamId)
	end

	self:run(connection)
end
function FuelSellTriggerEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.object)
	streamWriteInt32(streamId, self.triggerId)
	streamWriteBool(streamId, self.isFilling)
	if self.isFilling then
		writeNetworkNodeObject(streamId, self.trailer)
	end
end
function FuelSellTriggerEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(self, false, connection, self.object)
	end
	if self.object ~= nil then
		self.object:setIsFuelTankFilling(self.isFilling, self.trailer, true)
	end;
end
function FuelSellTriggerEvent.sendEvent(object, triggerId, isFilling, trailer, noEventSend)
	if (noEventSend == nil or noEventSend == false) then
		if g_server ~= nil then
			g_server:broadcastEvent(FuelSellTriggerEvent:new(object, triggerId, isFilling, trailer), nil, nil, object)
		else
			g_client:getServerConnection():sendEvent(FuelSellTriggerEvent:new(object, triggerId, isFilling, trailer))
		end
	end
end



